/*!
 * @file      UpdateManager.cpp
 * @brief     
 * @author    Jaehoon.Kim(jaehoon.kim@nhn.com)
 * @date      2012-03-28 
 * @warning   
 */
#include "StdAfx.h"
#include "UpdateManager.h"

CUpdateManager::CUpdateManager(void)
{
	m_bUpdatable = FALSE;
	m_IniURL = _T("");
	m_strCurrentVer = _T("");
	m_nTotalCnt = 0;
	m_bReboot = FALSE;
}

CUpdateManager::~CUpdateManager(void)
{
	UPDATEFILEINFO* pItem = NULL;
	for(size_t i=0; i<m_vecUpdateFileInfo.size(); i++)
	{
		pItem = m_vecUpdateFileInfo.at(i);
		if (pItem)
		{
			delete pItem;
			pItem = NULL;
		}		
	}

	for(size_t j=0; j<m_vecLocalFileInfo.size(); j++)
	{
		pItem = m_vecLocalFileInfo.at(j);
		if (pItem)
		{
			delete pItem;
			pItem = NULL;
		}		
	}
}

int CUpdateManager::Init(LPCTSTR pszIniFilePath, LPCTSTR pszDownloadUrl, LPCTSTR pszInstallPath)
{
	int result = 0;
	TCHAR szDrive[_MAX_DRIVE];
	TCHAR szDir[_MAX_DIR];
	TCHAR szFilename[_MAX_FNAME];
	TCHAR szExt[_MAX_EXT];
	CString strFileName;
	UPDATEFILEINFO updateFileInfo;
	CString strIniUrl;


	// ٿε ini   ּҸ ...
	if(pszIniFilePath == NULL)
	{
		result = ERROR_INVALID_PARAMETER;
		goto finish;
	}

	_tsplitpath(pszIniFilePath, szDrive, szDir, szFilename, szExt);
	strFileName = szFilename;
	strFileName += szExt;

	LoadLocalIniInfo(pszIniFilePath);

	strIniUrl = MakeUrl(pszDownloadUrl, strFileName);

	updateFileInfo.strDownloadFileName = CreateTempFileName();
	updateFileInfo.strFileName = strFileName;
	// ini ٿε ޴´..
	result = DownloadZipFile(strIniUrl, &updateFileInfo, NULL);
	if(result != 0)
		goto finish;

	// ini Ϸ  Ʈ  Ǵ  ִ  
	result = LoadUpdateFileInfo(updateFileInfo.strDownloadFileName);
	if(result == 1)
		goto finish;

	CopyFile(updateFileInfo.strDownloadFileName, pszIniFilePath, FALSE);
	DeleteFile(updateFileInfo.strDownloadFileName);


finish:
	return result;


}







CString CUpdateManager::MakeUrl(LPCTSTR pszUrl, LPCTSTR pszFileName)
{
	CString strUrl;

	strUrl = pszUrl;
	if(strUrl.Right(1)  != _T("/"))
	{
		strUrl += _T("/");
	}

	strUrl += pszFileName;


	return strUrl;
}


int CUpdateManager::DownloadZipFile(LPCTSTR pszURL, PUPDATEFILEINFO pFileInfo, HWND hwnd)
{
	HINTERNET hInternet = NULL, hURL = NULL;
	int result;
	HANDLE hFile = INVALID_HANDLE_VALUE;
	DWORD dwSize, dwRead, dwWritten, dwTotalWrittenBytes;
	BYTE baBuffer[ 1024 * 10 ];
	int nStatusCode;
	CString strDbg;
	
	if(pszURL == NULL)
	{
		result = ERROR_INVALID_PARAMETER;		
		goto finish;
	}

	hInternet = InternetOpen(_T("Ntalkbiz"), INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
	if( hInternet == NULL )
	{
		result = GetLastError();
		OutputDebugString(_T("InternetOpen()"));
		goto finish;
	}

	hURL = InternetOpenUrl(hInternet, pszURL, NULL, 0, INTERNET_FLAG_RELOAD, 0);
	if(hURL == NULL)
	{
		result = GetLastError();
		OutputDebugString(_T( "InternetOpenUrl()"));
		goto finish;
	}
	
	dwSize = sizeof(baBuffer);
	result = HttpQueryInfo(hURL, HTTP_QUERY_STATUS_CODE, baBuffer, &dwSize, NULL);
	if( result == FALSE )
	{
		result = GetLastError();
		OutputDebugString(_T( "HttpQueryInfo()"));
		goto finish;
	}

	nStatusCode = _ttoi((LPCTSTR)baBuffer);
	if( nStatusCode == 404 )
	{
		//
		// ã, ׳ ŵ
		//		
		result = ERROR_NOT_FOUND;
		strDbg.Format(_T("%s  ã  ϴ"), pszURL );
		OutputDebugString(strDbg);
		goto finish;
	}
	else if((nStatusCode != 200) && (nStatusCode != 304))
	{

		result = GetLastError();
		strDbg.Format(_T("κ %d  ޾ҽϴ" ), nStatusCode);
		OutputDebugString(strDbg);
		goto finish;
	}

	hFile = CreateFile(pFileInfo->strDownloadFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if( hFile == INVALID_HANDLE_VALUE )
	{
		CreateDirectory(pFileInfo->strFolder, NULL);
		hFile = CreateFile(pFileInfo->strDownloadFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile == INVALID_HANDLE_VALUE )
		{
			result = GetLastError();
			strDbg.Format(_T( "%s    ϴ" ), pFileInfo->strDownloadFileName);
			OutputDebugString(_T( "CreateFile() : "));
			OutputDebugString(strDbg);
			goto finish;
		}		
	}

	if( hwnd != NULL )
	{
		SendMessage(hwnd, WM_AGENTUPDATE_DOWNLOAD_BEGIN, pFileInfo->nSize, (LPARAM)( (LPCTSTR)pFileInfo->strFileName));
	}

	dwTotalWrittenBytes = 0;

	do
	{
		result = InternetQueryDataAvailable(hURL, &dwSize, 0, 0);
		if( result == FALSE )
		{
			result = GetLastError();
			OutputDebugString(_T( "InternetQueryDataAvailable()"));
			goto finish;
		}
		
		//
		// ۰ Ѿ  ؼ...
		//
		if( dwSize > sizeof(baBuffer) )
		{
			dwSize = sizeof(baBuffer);
		}

		result = InternetReadFile(hURL, baBuffer, dwSize, &dwRead);
		if(result == FALSE)
		{
			result = GetLastError();
			strDbg.Format(_T("InternetReadFile()"));
			OutputDebugString(strDbg);
			goto finish;
		}

		result = WriteFile(hFile, baBuffer, dwRead, &dwWritten, NULL);
		if(result == FALSE)
		{
			result = GetLastError();
			strDbg.Format(_T("WriteFile()"));
			OutputDebugString(strDbg);
			goto finish;
		}
			
		dwTotalWrittenBytes += dwWritten;
		if(hwnd != NULL)
		{
			SendMessage( hwnd, WM_AGENTUPDATE_DOWNLOAD_NBYTES, dwWritten, dwTotalWrittenBytes );
		}
	} while(dwRead != 0);

	if(hwnd != NULL)
	{
		SendMessage(hwnd, WM_AGENTUPDATE_DOWNLOAD_END, dwWritten, (LPARAM)( (LPCTSTR)pFileInfo->strFileName));
	}

	result = 0;

finish:
	if(result != 0)
	{
		OutputDebugString(_T( "CUpdateManager::DownloadFile()"));
	}

	if( hURL != NULL )
	{
		InternetCloseHandle( hURL );
	}

	if( hInternet != NULL )
	{
		InternetCloseHandle( hInternet );
	}
	
	if( hFile != INVALID_HANDLE_VALUE )
	{
		CloseHandle( hFile );
	}

	return result;
}


CString CUpdateManager::CreateTempFileName()
{
	TCHAR szTempPath[4096];
	TCHAR szTempFileName[8192];
	CString strResult;

	GetTempPath(sizeof(szTempPath) / sizeof(TCHAR), szTempPath);
	GetTempFileName(szTempPath, _T(""), 0, szTempFileName);

	strResult.Format( _T("Temp File Name : %s"), szTempFileName );
	OutputDebugString( strResult );

	strResult = szTempFileName;

	return strResult;
}


int CUpdateManager::LoadUpdateFileInfo(LPCTSTR pszIniFilePath)
{
	int result = 0;
	TCHAR szBuffer[4096];
	int nBufferSize = DIM(szBuffer);
	int nLen, nRegist;
	vector<CString> vecUpdateFiles;
	vector<CString>::iterator iter;
	PUPDATEFILEINFO pUpdateFileInfo;
	CString strVersion = _T("");
	
	m_dwTotlaBytes = 0;

	nLen = GetPrivateProfileString(_T("UpdateInfo"), _T("URL"), _T(""), szBuffer, nBufferSize, pszIniFilePath);	
	szBuffer[nLen] = _T('\0');
	m_IniURL = szBuffer;
	
	nLen = GetPrivateProfileString(_T("UpdateInfo"), _T("Version"), _T(""), szBuffer, nBufferSize, pszIniFilePath);	
	szBuffer[nLen] = _T('\0');
	strVersion = szBuffer;

	//ι 	
	if (IsCompareVersion(m_strCurrentVer, strVersion) == FALSE)
		return 1;
	

	nLen = GetPrivateProfileString(_T("UpdateInfo"), _T("Files"), _T(""), szBuffer, nBufferSize, pszIniFilePath);	
	szBuffer[nLen] = _T('\0');
	result = SplitString(szBuffer, _T(';'), vecUpdateFiles);

	if(vecUpdateFiles.empty() == true)
	{
		result = -1;
		m_bUpdatable = FALSE;
		
	}
	else
	{
		iter = vecUpdateFiles.begin();
		for(; iter != vecUpdateFiles.end(); iter++)
		{
			memset(szBuffer, 0x00, sizeof(TCHAR) * 4096);
			nLen = GetPrivateProfileString(*iter, _T("Version"), _T(""), szBuffer, nBufferSize, pszIniFilePath);			
			szBuffer[nLen] = _T('\0');
			strVersion = szBuffer;

			if(IsUpdateFile(*iter, strVersion) == false)
				continue;

			pUpdateFileInfo = new UPDATEFILEINFO;
			if(pUpdateFileInfo == NULL)
			{
				result = ERROR_NOT_ENOUGH_MEMORY;
				goto finish;
			}

			pUpdateFileInfo->strFileName = (*iter);

			nLen = GetPrivateProfileString(pUpdateFileInfo->strFileName, _T("Folder"), _T(""), szBuffer, nBufferSize, pszIniFilePath);
			szBuffer[nLen] = _T('\0');

			//    
			pUpdateFileInfo->strFolder = GetFullPath(szBuffer);

			//Ʈ   ũ
			nLen = GetPrivateProfileString(pUpdateFileInfo->strFileName, _T("Size"), _T(""), szBuffer, nBufferSize, pszIniFilePath);
			szBuffer[nLen] = _T('\0');
			pUpdateFileInfo->nSize = _tstoi(szBuffer);
			m_dwTotlaBytes += pUpdateFileInfo->nSize;

			//ڵ
			nLen = GetPrivateProfileString(pUpdateFileInfo->strFileName, _T("SelfRegister"), _T(""), szBuffer, nBufferSize, pszIniFilePath);
			szBuffer[nLen] = _T('\0');
			nRegist = _tstoi(szBuffer);
			pUpdateFileInfo->bSelfRegist = nRegist == 1? TRUE:FALSE;
	
			m_vecUpdateFileInfo.push_back(pUpdateFileInfo);
			m_nTotalCnt++;
		}
		m_bUpdatable = TRUE;
	}

finish:
	return result;
}


int CUpdateManager::LoadLocalIniInfo(LPCTSTR pszIniFilePath)
{
	int result = 0;
	TCHAR szBuffer[2048];
	int nBufferSize = DIM(szBuffer);
	int nLen;
	vector<CString> vecUpdateFiles;
	vector<CString>::iterator iter;
	PUPDATEFILEINFO pUpdateFileInfo;
	CString strVersion = _T("");

	nLen = GetPrivateProfileString(_T("UpdateInfo"), _T("Version"), _T(""), szBuffer, nBufferSize, pszIniFilePath);	
	szBuffer[nLen] = _T('\0');
	m_strCurrentVer = szBuffer;

	nLen = GetPrivateProfileString(_T("UpdateInfo"), _T("Files"), _T(""), szBuffer, nBufferSize, pszIniFilePath);	
	szBuffer[nLen] = _T('\0');
	result = SplitString(szBuffer, _T(';'), vecUpdateFiles);

	if(vecUpdateFiles.empty() == true)
	{
		result = -1;
	}
	else
	{
		iter = vecUpdateFiles.begin();
		for(; iter != vecUpdateFiles.end(); iter++)
		{
			pUpdateFileInfo = new UPDATEFILEINFO;
			if(pUpdateFileInfo == NULL)
			{
				result = ERROR_NOT_ENOUGH_MEMORY;
				goto finish;
			}

			m_vecLocalFileInfo.push_back(pUpdateFileInfo);

			pUpdateFileInfo->strFileName = (*iter);

			nLen = GetPrivateProfileString(pUpdateFileInfo->strFileName, _T("Version"), _T(""), szBuffer, nBufferSize, pszIniFilePath);			
			szBuffer[nLen] = _T('\0');			
			pUpdateFileInfo->strVersion = szBuffer;

		}

	}



finish:
	return result;	
}


/*!
 * @brief     ڿ иؼ Ϳ ´
 * @return    int
 * @param     LPCTSTR pszSrc
 * @param     TCHAR split
 * @param     std::vector<CString> & rvec
 * @author    Jaehoon.Kim(jaehoon.kim@nhn.com)
 * @date      2012-03-26 
 * @warning   
 */
int CUpdateManager::SplitString(LPCTSTR pszSrc, TCHAR split, vector<CString> &rvec)
{
	CString strSrc, strTokenize;
	int nCurPos = 0;

	strSrc = pszSrc;

	strTokenize = strSrc.Tokenize(&split, nCurPos);
	while(strTokenize != _T(""))
	{
		rvec.push_back(strTokenize);
		strTokenize = strSrc.Tokenize(&split, nCurPos);
	}

	return 0;

}


CString CUpdateManager::GetFullPath(CString strFolder)
{
	TCHAR szBuffer[4096];

	GetCurrentDirectory(DIM(szBuffer), szBuffer);
	strFolder.Replace(_T("%CURRENTDIR%"), szBuffer);

	return strFolder;
}
int CUpdateManager::DownloadUpdateFiles(HWND hWnd)
{
	size_t nSize = m_vecUpdateFileInfo.size();

	CString strIniUrl = _T("");
	CString strTmp = _T("");
	UPDATEFILEINFO* pInfo = NULL;
	size_t i=0;
	for (i = 0; i < nSize; i++)
	{
		pInfo = m_vecUpdateFileInfo.at(i);
		if (pInfo)
		{
			TCHAR szBuffer[4096];
			GetCurrentDirectory(DIM(szBuffer), szBuffer);
			strTmp = pInfo->strFolder;
			strTmp.Replace(szBuffer, _T(""));
			if(strTmp.Left(1)  == _T("/"))
			{
				strTmp = strTmp.Right(strTmp.GetLength()-1);
			}
			strIniUrl = MakeUrl(m_IniURL+strTmp, pInfo->strFileName);

			pInfo->strDownloadFileName = CreateTempFileName();
			if(DownloadZipFile(strIniUrl, pInfo, hWnd) != 0)
				AfxMessageBox(pInfo->strFileName);
		}
	}

	// tmpϷ ٿް  丮 
	CString strFileName = _T("");
	for (i = 0; i < nSize; i++)
	{
		pInfo = m_vecUpdateFileInfo.at(i);
		if (pInfo)
		{
			strFileName = MakeUrl(pInfo->strFolder, pInfo->strFileName);
			if(!CopyFile(pInfo->strDownloadFileName, strFileName, FALSE))
			{		
				DWORD dwErr = GetLastError();
				if(dwErr == ERROR_SHARING_VIOLATION)
				{
					CopyFile(pInfo->strDownloadFileName, strFileName + _T(".nts"), FALSE);
					int result = MoveFileEx(strFileName + _T(".nts"), strFileName, MOVEFILE_DELAY_UNTIL_REBOOT | MOVEFILE_REPLACE_EXISTING);
					if(result == FALSE)
					{
						CString strMsg;
						strMsg.Format(_T("%s    ϴ."), strFileName);
						AfxMessageBox(strMsg);
					}
					else
					{
						m_bReboot = TRUE;
					}
				}
				else
				{
					CreateDirectory(pInfo->strFolder, NULL);
					CopyFile(pInfo->strDownloadFileName, strFileName, FALSE);					
				}

				DeleteFile(pInfo->strDownloadFileName);
			}

			if (pInfo->bSelfRegist)
			{
				strTmp.Format(_T("regsvr32 /s /c %s"), strFileName);
				ShellExecute(0, 0, strTmp, 0, 0, SW_SHOWNORMAL);
			}
		}
	}
	
	return 0;
}


//    ؼ Ʈ  θ Ǵ
BOOL CUpdateManager::IsUpdateFile(CString strFileName, CString strVersion)
{
	BOOL isUpdate = TRUE;
	UPDATEFILEINFO* pItem = NULL;
	for (size_t i=0; i<m_vecLocalFileInfo.size(); i++)
	{
		pItem = m_vecLocalFileInfo.at(i);
		if (pItem)
		{
			if(pItem->strFileName == strFileName)
			{
				isUpdate = IsCompareVersion(pItem->strVersion, strVersion);	
				break;
			}
		}
	}

	return isUpdate;
}


/*!
 * @brief      Ѵ.
 * @return    BOOL : newVersion  ũ TRUE, ׿ܴ FALSE
 * @param     LPCTSTR strOldVersion
 * @param     LPCTSTR strNewVersion
 * @author    Jaehoon.Kim(jaehoon.kim@nhn.com)
 * @date      2012-03-28 
 * @warning   
 */
BOOL CUpdateManager::IsCompareVersion(LPCTSTR strOldVersion, LPCTSTR strNewVersion)
{
	BOOL bLargeNewVersion = FALSE;
	int nOldMajor, nNewMajor;
	int nOldMinor, nNewMinor;
	std::vector<CString> vecOldVersion;
	std::vector<CString> vecNewVersion;

	SplitString(strOldVersion, _T('.'), vecOldVersion);
	SplitString(strNewVersion, _T('.'), vecNewVersion);
	
	nOldMajor = _wtoi(vecOldVersion[0].GetBuffer());	
	nNewMajor = _wtoi(vecNewVersion[0].GetBuffer());	

	if(nOldMajor < nNewMajor)
		bLargeNewVersion = TRUE;
	else if(nOldMajor == nNewMajor)
	{		
		nOldMinor = _wtoi(vecOldVersion[1].GetBuffer());
		nNewMinor = _wtoi(vecNewVersion[1].GetBuffer());
		if(nOldMinor < nNewMinor)
			bLargeNewVersion = TRUE;		
	}

	return bLargeNewVersion;
}

BOOL CUpdateManager::IsReboot()
{
	return m_bReboot;
}